/*
 * exception.c
 *
 *	Christophe Lucas <clucas@rotomalug.org>
 *
 */

#include <testos/types.h>
#include <testos/errno.h>
#include <x86/exception.h>
#include <x86/irq.h>
#include <x86/idt.h>

#define EXCEPTION_BASE		0
#define EXCEPTION_NR		32	

extern vaddr_t exception_wrapper_array[EXCEPTION_NR];
exception_handler_t exception_handler_array[EXCEPTION_NR] = { NULL, };

ret_t exception_setup_routine(int index, exception_handler_t routine)
{
	int flags;
	if ((index < 0) || (index>EXCEPTION_NR)) 
		return -EINVAL;

	disable_IRQs(flags);
	exception_handler_array[index] = routine ;
	if( routine != NULL) {
		int retval;
		retval =  set_idt_handler(EXCEPTION_BASE+index, (vaddr_t)exception_wrapper_array[index], 0);	
		if( retval != OK ) {
			return -EINVAL;
		}
	} else {
		set_idt_handler(EXCEPTION_BASE+index, (vaddr_t)NULL,0);
	}
	restore_IRQs(flags);

	return OK;
}

exception_handler_t get_exception_routine( int exception_nb )
{
	if ((exception_nb < 0) || (exception_nb>=EXCEPTION_NR)) 
		return NULL;

	return exception_handler_array[exception_nb];
}

ret_t setup_exceptions()
{
	set_idt_handler(EXCEPTION_BASE + EXCEPT_INVALID_TSS, 
				(vaddr_t) exception_wrapper_array[EXCEPT_INVALID_TSS], 0 );

	return set_idt_handler(EXCEPTION_BASE + EXCEPT_DOUBLE_FAULT, 
				(vaddr_t) exception_wrapper_array[EXCEPT_DOUBLE_FAULT], 0 );
}


